/*
CandlePredTest_adaptive.mq4
 2009 Eugene Logunov (lea)
*/
#property copyright " 2009 Eugene Logunov (lea)"

/***************************************************************************************/

// defining the possible candlestick directions
#define DIRECTION_NONE 0
#define DIRECTION_UP 1
#define DIRECTION_DOWN -1
// defining the model states
#define STATE_NORMAL 0
#define STATE_REVERSE 1

/***************************************************************************************/

// magic of the positions being opened
extern int ordersMagic=1001110;
// volume of the positions being opened
extern double lots=0.1;
// initial model state (see the explanation in the article)
extern int initialModelState=STATE_NORMAL;

/***************************************************************************************/

// opening time of the last processed bar
datetime expTime;

// check for the new bar
bool IsNewBar() {
   if (expTime==Time[0]) {
      return (false);
   }
   expTime=Time[0];
   return (true);
}

/***************************************************************************************/

// getting the candlestick direction
double GetCandleDirection(int shift) {
   double diff=Close[shift]-Open[shift];
   if (diff>0.0) {
      return (DIRECTION_UP);
   }
   else if (diff<0.0) {
      return (DIRECTION_DOWN);
   }
   else /*if (diff==0.0)*/ {
      return (DIRECTION_NONE);
   }
}

/***************************************************************************************/

// model state
int modelState;

// forecast based on the current model
int ModelPredict() {
   if (modelState==STATE_NORMAL) {
      return (GetCandleDirection(1));
   }
   else /*if (modelState==STATE_REVERSE)*/ {
      return (-GetCandleDirection(1));
   }
}

// updating the model state
void ModelUpdate(bool correct) {
   // if the forecast was erroneous, change the model state
   if (!correct) {
      if (modelState==STATE_NORMAL) {
         modelState=STATE_REVERSE;
      }
      else /*if (modelState==STATE_REVERSE)*/ {
         modelState=STATE_NORMAL;
      }
   }
}

/***************************************************************************************/

// opening one position in the specified direction
void TradeOpenPosition(int direction,string comment) {
   switch (direction) {
      case DIRECTION_UP:
         OrderSend(Symbol(),OP_BUY,lots,Ask,3,0,0,comment,ordersMagic,0,CLR_NONE);
         break;
      case DIRECTION_DOWN:
         OrderSend(Symbol(),OP_SELL,lots,Bid,3,0,0,comment,ordersMagic,0,CLR_NONE);
         break;
   }
}

// closing all positions in the specified direction
void TradeClosePositions(int direction) {
   for (int i=OrdersTotal()-1;i>=0;i--) {
      if (!OrderSelect(i,SELECT_BY_POS,MODE_TRADES)) {
         continue;
      }
      if (OrderSymbol()!=Symbol()) {
         continue;
      }
      if (OrderMagicNumber()!=ordersMagic) {
         continue;
      }
      if ((direction==DIRECTION_DOWN)&&(OrderType()==OP_SELL)) {
         OrderClose(OrderTicket(),OrderLots(),Ask,3,CLR_NONE);
      }
      if ((direction==DIRECTION_UP)&&(OrderType()==OP_BUY)) {
         OrderClose(OrderTicket(),OrderLots(),Bid,3,CLR_NONE);
      }
   }
}

/***************************************************************************************/

// previous forecast of the candlestick direction
int prediction=DIRECTION_NONE;

// initialization of the Expert Advisor
int init() {
   // set the model to the specified state
   modelState=initialModelState;
   return (0);
}

// working function of the Expert Advisor
int start() {
   // positions are opened once on each bar
   if (!IsNewBar()) {
      return (0);
   }
   // close all open positions
   TradeClosePositions(DIRECTION_UP);
   TradeClosePositions(DIRECTION_DOWN);
   // update the model state
   if (prediction!=DIRECTION_NONE) {
      ModelUpdate(prediction==GetCandleDirection(1));
   }
   // predict the color of the next candlestick
   prediction=ModelPredict();
   if (prediction==DIRECTION_NONE) {
      return (0);
   }
   // open a position in the right direction
   TradeOpenPosition(prediction,"");
   return (0);
}

